import axios from "axios"
import { EStatuses } from "../enums/EStatuses"
import IProduct from "../interfaces/IProduct"
import IResponse from "../interfaces/IResponse"
import { instance } from "./instance"


class ProductApi {

    public getProducts = async(): Promise<IResponse<IProduct[] | undefined>> => {
        try {
            const response = await instance.get('/products')
            return response.data
        } catch (err: unknown) {
            const error = err as Error
            const response: IResponse = {
                status: EStatuses.NOT_OK,
                result: undefined,
                message: error.message
            }
            return response
        }
    }

    public getProductById = async(id: string): Promise<IResponse<IProduct | undefined>> => {
        try {
            const response = await instance.get(`/products/${id}`)
            return response.data
        } catch (err: unknown) {
            const error = err as Error
            const response: IResponse = {
                status: EStatuses.NOT_OK,
                result: undefined,
                message: error.message
            }
            return response
        }
    }

    public createProduct = async(product: FormData): Promise<IResponse> => {
        try {
            const response = await instance.post(`/products`, product)
            return response.data
        } catch (err: unknown) {
            const error = err as Error
            const response: IResponse = {
                status: EStatuses.NOT_OK,
                result: undefined,
                message: error.message
            }
            return response
        }
    }

    public deleteProductById = async(id: string): Promise<IResponse> => {
        try {
            const response = await instance.delete(`/products/${id}`)
            return response.data
        } catch (err: unknown) {
            const error = err as Error
            const response: IResponse = {
                status: EStatuses.NOT_OK,
                result: undefined,
                message: error.message
            }
            return response
        }
    }
}

export const productApi = new ProductApi()