import React, { ChangeEvent, FormEvent, FunctionComponent, ReactElement, useState } from "react";
import { useDispatch } from "react-redux";
import IProductDto from "../../interfaces/IProductDto";
import { createProduct } from "../../store/products/products.slice";
import { AppDispatch } from "../../store/store";
import styles from './ProductForm.module.css'


const ProductForm: FunctionComponent = (): ReactElement => {
    const dispatch: AppDispatch = useDispatch()
    const [product, setProduct] = useState<IProductDto>({
        product: '',
        price: 0,
        description: '',
        image: undefined
    })
    const [fileName, setFileName] = useState<string>('')
    const inputHandler = (e: ChangeEvent<HTMLInputElement>): void => {
        setProduct(prevState => {
            return {...prevState, [e.target.name]: e.target.value}
        })
        console.log(product)
    }
    const inputFileHandler = (e: ChangeEvent<HTMLInputElement>): void => {
        setProduct(prevState => {
            return {...prevState, 
                image: e.target.files ? e.target.files[0] : undefined}
        })
        setFileName(e.target.files && e.target.files[0] ? e.target.files[0].name : '')
    }
    const submitHandler = (e: FormEvent) => {
        e.preventDefault()

        const formData = new FormData()
        Object.keys(product).forEach((key: any) => {
            //@ts-ignore
            formData.append(key, product[key])
        })

        dispatch(createProduct(formData))
    }
    return (
        <div className={styles.ProductForm}>
            <form onSubmit={submitHandler}>
                <input 
                    name={'product'} 
                    type="text" 
                    placeholder="Title" 
                    value={product.product}
                    onChange={inputHandler}
                />
                <input 
                    name={'price'} 
                    type="number" 
                    placeholder="Price" 
                    value={product.price}
                    onChange={inputHandler}
                />
                <input 
                    name={'description'} 
                    type="text" 
                    placeholder="Description" 
                    value={product.description}
                    onChange={inputHandler}
                />
                <label>
                    <input 
                        className={styles.ProductForm__fileInput}
                        type="file" 
                        name={'image'} 
                        onChange={inputFileHandler} 
                    />
                    <h2>
                        CHOOSE FILE
                    </h2>
                    <span>{fileName}</span>
                    
                </label>
                <button>SEND</button>
            </form>
        </div>
    )
}

export default ProductForm