import { createSlice } from '@reduxjs/toolkit'
import { productApi } from '../../api/productApi'
import IProduct from '../../interfaces/IProduct'
import IProductDto from '../../interfaces/IProductDto'
import IResponse from '../../interfaces/IResponse'
import { createAppAsyncThunk } from '../createAppAsyncThunk'
import IProductsState from './IProductsState'

const namespace = 'products'

export const getProducts = createAppAsyncThunk(
    `${namespace}/getProducts`,
    async () => {
        return productApi.getProducts()
    }
)

export const getProductById = createAppAsyncThunk(
    `${namespace}/getProductById`,
    async (id: string): Promise<IResponse<IProduct | undefined>> => {
        return productApi.getProductById(id)
    }
)

export const createProduct = createAppAsyncThunk(
    `${namespace}/createProduct`,
    async (product: FormData) => {
        return productApi.createProduct(product)
    }
)

export const deleteProductById = createAppAsyncThunk(
    `${namespace}/deleteProductById`,
    async (id: string) => {
        return productApi.deleteProductById(id)
    }
)

export const productsSlice = createSlice({
    name: namespace,
    initialState: {
        products: [] as IProduct[],
        product: {},
        loadingProducts: false,
        messageProducts: ''
    } as IProductsState,
    reducers: {},
    extraReducers: (builder) => {
        builder
            .addCase(getProducts.rejected, (state) => {
                state.loadingProducts = false
            })
            .addCase(getProducts.pending, (state) => {
                state.loadingProducts = true
            })
            .addCase(getProducts.fulfilled, (state, action) => {
                state.loadingProducts = false
                state.products = action.payload.result as IProduct[]
                state.messageProducts = action.payload.message
            })
            .addCase(getProductById.rejected, (state) => {
                state.loadingProducts = false
            })
            .addCase(getProductById.pending, (state) => {
                state.loadingProducts = true
            })
            .addCase(getProductById.fulfilled, (state, action) => {
                state.loadingProducts = false
                state.product = action.payload.result as IProduct
                state.messageProducts = action.payload.message
            })
            .addCase(createProduct.rejected, (state) => {
                state.loadingProducts = false
            })
            .addCase(createProduct.pending, (state) => {
                state.loadingProducts = true
            })
            .addCase(createProduct.fulfilled, (state, action) => {
                state.loadingProducts = false
                state.messageProducts = action.payload.message
            })
            .addCase(deleteProductById.rejected, (state) => {
                state.loadingProducts = false
            })
            .addCase(deleteProductById.pending, (state) => {
                state.loadingProducts = true
            })
            .addCase(deleteProductById.fulfilled, (state, action) => {
                state.loadingProducts = false
                state.messageProducts = action.payload.message
            })
    }
})